<?php
/*-------------------------------------------------------+
| PHPFusion Content Management System
| Copyright (C) PHP Fusion Inc
| https://phpfusion.com/
+--------------------------------------------------------+
| Filename: classes/admin_category.php
| Author: karrak
+--------------------------------------------------------+
| This program is released as free software under the
| Affero GPL license. You can redistribute it and/or
| modify it under the terms of this license which you
| can read by viewing the included agpl.txt or online
| at www.gnu.org/licenses/agpl.html. Removal of this
| copyright header is strictly prohibited without
| written permission from the original author(s).
+--------------------------------------------------------*/
namespace PHPFusion\TLine;

class TLineAdmin extends TLineServer {
    private static $instance = NULL;
    public $tlinedata = NULL;

    public function __construct() {
        parent::__construct();
    }

    public static function getInstance() {
        if (self::$instance == NULL) {
            self::$instance = new static();
        }
        return self::$instance;
    }

    public function startAdmin() {
        if ( check_get( 'cancel' ) ) {
            redirect( FUSION_SELF.fusion_get_aidlink() );
        }
        if ( check_get( 'ref' ) && get( 'ref' ) == "timelineform" ) {
            $this->timelineForm();
        } else {
            self::timelineQuery();
            $this->timelinelisting();
        }
    }

    private function timelineQuery() {
        $timeline = [];
        $result = dbquery( $this->sqlTimeline->getTimelineQuery( ['order' => "timeline_datestamp DESC"] ) );
        if ( dbrows( $result ) ) {
            while ( $data = dbarray( $result ) ) {
                $timeline[$data['timeline_datestamp']] = $data;
            }
        }
        krsort( $timeline );
        foreach ( $timeline as $timestamp => $item ) {
            $format = showdate( 'Y M j', $timestamp );
            $this->tlinedata[$format][] = $item;
        }
    }

    private function timelinelisting() {
        echo "<a class='btn btn-success btn-sm m-r-10' href='" . clean_request( "ref=timelineform", ["ref"], FALSE ) . "'><i class='fa fa-fw fa-plus m-r-10'></i>" . self::$locale['TMLN_041'] . "</a>";
        if ( !empty( $this->tlinedata ) ) {
            echo "<div class='table-responsive'>";
                echo "<table id='links-table' class='table table-striped'>";
                    foreach ( $this->tlinedata as $timestamp => $item ) {
                        echo "<tr>";
                            echo "<td class='text-center' colspan='3'>".$timestamp."</td>";
                        echo "</tr>";
                        foreach ( $item as $idata ) {
                            $edit_link = clean_request( 'section=timeline&ref=timelineform&action=edit&timeline_id=' . $idata['timeline_id'], ['section', 'ref', 'action', 'timeline_id'], FALSE );
                            $delete_link = clean_request( 'section=timeline&ref=timelineform&action=delete&timeline_id=' . $idata['timeline_id'], ['section', 'ref', 'action', 'timeline_id'], FALSE );
                            echo "<tr>";
                                echo "<td>" . $idata['timeline_title'] . "</td>";
                                echo "<td>" . trimlink( $idata['timeline_description'], 100 ) . "</td>";
                                echo "<td>
                                    <a href='" . $edit_link . "' title=" . self::$locale['edit'] . ">" . self::$locale['edit'] . "</a>&nbsp;|&nbsp;
                                    <a href='" . $delete_link . "' title=" . self::$locale['delete'] . " onclick=\"return confirm('" . self::$locale['TMLN_042'] . "')\">" . self::$locale['delete'] . "</a>
                                </td>";
                            echo "</tr>";
                        }
                    }
                echo "</table>";
            echo "</div>";
        }

    }

    private function timelineForm() {
        $data = [];
        if ( check_post( 'save_cat' ) ) {
            $savetline = [
                'timeline_id'          => sanitizer( 'timeline_id', '', 'timeline_id' ),
                'timeline_user'        => sanitizer( 'timeline_user', 0, 'timeline_user' ),
                'timeline_title'       => sanitizer( 'timeline_title', '', 'timeline_title' ),
                'timeline_description' => sanitizer( addslashes( 'timeline_description'), '', 'timeline_description' ),
                'timeline_datestamp'   => ( check_post( 'update_datestamp' ) ? time() : sanitizer( 'timeline_datestamp', '', 'timeline_datestamp' ) ),
                'timeline_language'    => sanitizer( ['timeline_language'], LANGUAGE, 'timeline_language' )
            ];
            if ( fusion_safe() ) {
                dbquery_insert( DB_TIMELINE, $savetline, empty( $savetline['timeline_id'] ) ? 'save' : 'update' );
                addnotice( "success", empty( $savetline['timeline_id'] ) ? self::$locale['TMLN_048'] : self::$locale['TMLN_048'] );
                redirect( clean_request( "", ['ref', 'action', 'timeline_id'], FALSE ) );
            }
        }

        if ( ( check_get( 'action' ) && get( 'action' ) == "edit" ) && ( check_get( 'timeline_id' ) && get( 'timeline_id', FILTER_VALIDATE_INT ) ) ) {
            $result = dbquery( "SELECT *
                FROM " . DB_TIMELINE . "
                " . ( multilang_table( "TMLN" ) ? "WHERE " . in_group( 'timeline_language', LANGUAGE ) . " AND" : "WHERE" ) . " timeline_id = :timelineid", [':timelineid' => get( 'timeline_id' )]
            );
            if ( dbrows( $result ) ) {
                $data = dbarray( $result );
            } else {
                redirect( clean_request( '', ['action'], FALSE ) );
            }
        } else if ( ( check_get( 'action' ) && get( 'action' ) == "delete" ) && ( check_get( 'timeline_id' ) && get( 'timeline_id', FILTER_VALIDATE_INT ) ) ) {
            dbquery( "DELETE FROM  " . DB_TIMELINE . " WHERE timeline_id = :timelineid", [':timelineid' => get( 'timeline_id' )] );
            addnotice( "success", self::$locale['TMLN_050'] );
            redirect( clean_request( '', ['ref', 'action', 'timeline_id'], FALSE ) );
        }
        $data += $this->def_timeline;
        // Textarea Settings
        if ( !fusion_get_settings( "tinymce_enabled" ) ) {
            $tlineExtendedSettings = [
                'required'    => TRUE,
                'preview'     => TRUE,
                'html'        => TRUE,
                'autosize'    => TRUE,
                'placeholder' => self::$locale['TMLN_043'],
                'form_name'   => 'addtimeline',
                'wordcount'   => TRUE,
                'path'        => IMAGES_T
            ];
        } else {
            $tlineExtendedSettings = [
                'required'      => TRUE,
                'type'          => fusion_get_settings( 'tinymce_enabled' ) ? 'tinymce' : 'html',
                'tinymce'       => fusion_get_settings( 'tinymce_enabled' ) && iADMIN ? 'advanced' : 'simple',
                'tinymce_image' => FALSE,
                'path'          => IMAGES_T,
                'form_name'     => 'addtimeline'
            ];
        }

        echo openform( 'addtimeline', 'post', FUSION_REQUEST, ['enctype' => TRUE, 'class' => 'm-t-20'] );
        openside('');
        echo form_hidden( 'timeline_id', '', $data['timeline_id'] );
        echo form_hidden( 'timeline_datestamp', '', $data['timeline_datestamp'] );
        echo form_text( 'timeline_title', self::$locale['TMLN_044'], $data['timeline_title'], [
            'required'    => TRUE,
            'placeholder' => self::$locale['TMLN_044'],
            'error_text'  => self::$locale['TMLN_045']
        ] );
        echo form_textarea( 'timeline_description', self::$locale['TMLN_043'], $data['timeline_description'], $tlineExtendedSettings );
        echo form_datepicker( 'timeline_datestamp', self::$locale['TMLN_046'], $data['timeline_datestamp'], [
            'width' => '100%'
        ] );
        if ( get( 'action' ) == 'edit' ) {
            echo form_checkbox( 'update_datestamp', self::$locale['TMLN_047'], 0, [
                'toggle' => TRUE
            ] );
        }
        if ( multilang_table( 'TMLN' ) ) {
            echo form_select( 'timeline_language[]', self::$locale['language'], $data['timeline_language'], [
                'options'     => fusion_get_enabled_languages(),
                'placeholder' => self::$locale['choose'],
                'width'       => '100%',
                'multiple'    => TRUE
            ] );
        } else {
            echo form_hidden( 'timeline_language', '', $data['timeline_language'] );
        }
        echo form_select( 'timeline_user', 'Mentes felhasznalokent', 0, [
                'options'  => self::tlineUserData(),
                'ext_tip' => 'hagy uresen ha nem felhasznalonak megy'
            ] );
        echo form_button( 'save_cat', self::$locale['save'], self::$locale['save'], ['class' => 'btn-success', 'icon' => 'fa fa-hdd-o'] );
        echo closeform();
        closeside();
    }
}
