<?php
/*-------------------------------------------------------+
| PHPFusion Content Management System
| Copyright (C) PHP Fusion Inc
| https://phpfusion.com/
+--------------------------------------------------------+
| Filename: classes/timeline.php
| Author: karrak
+--------------------------------------------------------+
| This program is released as free software under the
| Affero GPL license. You can redistribute it and/or
| modify it under the terms of this license which you
| can read by viewing the included agpl.txt or online
| at www.gnu.org/licenses/agpl.html. Removal of this
| copyright header is strictly prohibited without
| written permission from the original author(s).
+--------------------------------------------------------*/
namespace PHPFusion\TLine;

class TimeLine extends TLineServer {
    private static $instance = NULL;
    private $allowed_filters = ['website', 'user'];
    private $type;
    public $info = [];

    public function __construct() {
        parent::__construct();
        add_to_title( \PHPFusion\SiteLinks::get_current_SiteLinks( "", "link_name" ) );
    }

    public static function getInstance() {
        if (self::$instance === NULL) {
            self::$instance = new static();
        }
        return self::$instance;
    }

    public function startTimeline() {
        set_title( self::$locale['TMLN_002'] );

        $tlinetab['title'][] = self::$locale['TMLN_011'];
        $tlinetab['id'][] = "website";
        $tlinetab['icon'][] = "fa fa-fw fa-globe m-r-10";
        if ( !empty( $this->timelinest['tline_user'] ) ) {
            $tlinetab['title'][] = self::$locale['TMLN_012'];
            $tlinetab['id'][] = "user";
            $tlinetab['icon'][] = "fa fa-fw fa-user m-r-10";
        }

        $section = get( 'section', FILTER_DEFAULT );
        $section = isset( $section ) && in_array( $section, $this->allowed_filters ) ? $section : $this->allowed_filters[0];

        opentable( self::$locale['TMLN_010'] );
        echo opentab( $tlinetab, $section, 'tline', TRUE, '', 'section', ['rowstart', 's_action'] );
        $info = [
            'line' => $this->tline[$this->timelinest['tline_line']]
        ];
        switch ( $section ) {
            case 'user':
                if ( !empty( $this->timelinest['tline_user'] ) ) {
                    $info += [
                        'tablename'  => check_post( 'userid' ) ? fusion_get_username( post( 'userid' ) ). self::$locale['TMLN_013'] : self::$locale['TMLN_014'],
                        'selectuser' => self::userSelect(),
                        'userline'   => self::userTimeline( check_post('userid') ? post('userid') : fusion_get_userdata( 'user_id' ) )
                    ];
                }
                break;
            default:
                $info += [
                    'tablename' => self::$locale['TMLN_011'],
                    'ittem'     => self::timeline()
                ];
        }
        displayTimeline( $info );
        echo closetab();
        closetable();
        //print_p($info);
    }

    private static function userSelect() {
        $info = [
            'openform'  => openform( 'timeline_user', 'post', fusion_get_settings( 'site_seo' ) ? PERMALINK_CURRENT_PATH : FUSION_REQUEST ),
            'select'    => form_select( 'userid', '', ( check_post( 'userid' ) ? post( 'userid' ) : fusion_get_userdata( 'user_id' ) ), [
                'options'  => self::tlineUserData(),
                'onchange' => 'document.timeline_user.submit()'
            ] ),
            'closeform' => closeform()
    	];
    	return $info;
    }

    private function timeline() {
        $timeline = [];
        //$_timeline = [];
        $_timeline['nodata'] = '';
        if (!empty( $this->timelinest['tline_autouser'] ) ) {
            $result_u = dbquery( "SELECT user_id, user_name, user_status, user_avatar, user_joined
                FROM " . DB_USERS . "
                ORDER BY user_joined DESC
            ");
            if ( dbrows( $result_u ) ) {
                while ( $data = dbarray( $result_u ) ) {
                    $timeline[$data['user_joined']] = [
                        'title'       => sprintf( self::$locale['TMLN_096'], $data['user_name'] ),
                        'description' => sprintf( self::$locale['TMLN_097'], $data['user_name'] )
                    ];
                }
            }
        }

        $result = dbquery( $this->sqlTimeline->getTimelineQuery( ['condition' => "timeline_user = '0'", 'order' => "timeline_datestamp DESC"] ) );
        if ( dbrows( $result ) ) {
            while ( $data = dbarray( $result ) ) {
                $timeline[$data['timeline_datestamp']] = [
                    'title'       => !empty( $data['timeline_title'] ) ? $data['timeline_title'] : '',
                    'description' => !empty( $data['timeline_description'] ) ? $data['timeline_description'] : ''
                ];
            }
        }

        if ( !empty( $timeline ) ) {
            krsort( $timeline );
            foreach ( $timeline as $timestamp => $item ) {
                $format = showdate( 'Y M j', $timestamp );
                $_timeline['data'][$format][] = $item;
            }
        } else {
            $_timeline['nodata'] = self::$locale['TMLN_016'];
        }
        return $_timeline;
    }

    private function userTimeline( $userid = NULL ) {
        $_timeline = [];
        $timeline = [];
        if ( $userid > 0 && !empty( $this->timelinest['tline_user'] ) ) {
            $tlinecat = [];
            $max_items = 50;
            $limit = $max_items / 5;
            $condition = self::timelineInfusion();

            $comments = dbquery( "SELECT c.*, u.user_id, u.user_name, u.user_status, u.user_avatar
                FROM ".DB_COMMENTS." AS c
                LEFT JOIN ".DB_USERS." AS u ON u.user_id = c.comment_name
                WHERE c.comment_name = '" . $userid . "' AND c.comment_hidden = 0 AND ($condition)
                ORDER BY c.comment_datestamp DESC
            " );
            while ( $data = dbarray( $comments ) ) {
                $comments_per_page = fusion_get_settings( 'comments_per_page' );
                $c_data = [];
                switch ( $data['comment_type'] ) {
                    case 'A':
                        if ( defined( 'ARTICLES_EXISTS' ) ) {
                            $result_a = dbquery( "SELECT a.article_subject
                                FROM " . DB_ARTICLES . " AS a
                                LEFT JOIN " . DB_ARTICLE_CATS . " AS ac ON ac.article_cat_id = a.article_cat
                                WHERE a.article_id = :id AND a.article_draft = 0 AND " . groupaccess( 'a.article_visibility' ) . ( multilang_table( 'AR' ) ? " AND " . in_group( 'ac.article_cat_language', LANGUAGE ) : '' ) . "
                                ORDER BY a.article_datestamp DESC", [':id' => $data['comment_item_id']]
                            );
                            if ( dbrows( $result_a ) ) {
                                $articles_data = dbarray( $result_a );
                                $comment_start = dbcount( '(comment_id)', DB_COMMENTS, "comment_item_id = '" . $data['comment_item_id'] . "' AND comment_type = 'A' AND comment_id <= " . $data['comment_id'] );
                                $comment_start = $comment_start > $comments_per_page ? '&c_start_news_comments=' . ( ( floor( $comment_start / $comments_per_page ) * $comments_per_page ) - $comments_per_page ) : '';
                                $c_data += [
                                    'type'  => self::$locale['TMLN_090'],
                                    'link'  => INFUSIONS . 'articles/articles.php?article_id=' . $data['comment_item_id'] . $comment_start . '#c' . $data['comment_id'],
                                    'title' => $articles_data['article_subject']
                                ];
                            }
                        }
                        break;
                    case 'B':
                        if ( defined( 'BLOG_EXISTS' ) ) {
                            $result_b = dbquery( "SELECT b.blog_subject
                                FROM " . DB_BLOG . " AS b
                                LEFT JOIN " . DB_BLOG_CATS . " AS bc ON bc.blog_cat_id = b.blog_cat
                                WHERE b.blog_id = :id AND (b.blog_start = 0 OR b.blog_start <= '" . time() . "') AND (b.blog_end = 0 OR b.blog_end >= '" . time() . "') AND b.blog_draft = 0 AND " . groupaccess( 'b.blog_visibility' ) . ( multilang_table( 'BL' ) ? " AND " . in_group( 'b.blog_language', LANGUAGE ) : '' ) . "
                                ORDER BY b.blog_datestamp DESC", [':id' => $data['comment_item_id']]
                            );
                            if ( dbrows( $result_b ) ) {
                                $blog_data = dbarray( $result_b );
                                $comment_start = dbcount( '(comment_id)', DB_COMMENTS, "comment_item_id = '" . $data['comment_item_id'] . "' AND comment_type = 'B' AND comment_id <= " . $data['comment_id'] );
                                $comment_start = $comment_start > $comments_per_page ? '&c_start_news_comments=' . ( ( floor( $comment_start / $comments_per_page ) * $comments_per_page ) - $comments_per_page ) : '';
                                $c_data += [
                                    'type'  => self::$locale['TMLN_091'],
                                    'link'  => INFUSIONS . 'blog/blog.php?readmore=' . $data['comment_item_id'] . $comment_start . '#c' . $data['comment_id'],
                                    'title' => $blog_data['blog_subject']
                                ];
                            }
                        }
                        break;
                    case 'D':
                        if ( defined( 'DOWNLOADS_EXISTS' ) ) {
                            $result_d = dbquery( "SELECT d.download_title
                                FROM " . DB_DOWNLOADS . " AS d
                                INNER JOIN " . DB_DOWNLOAD_CATS . " AS dc ON dc.download_cat_id = d.download_cat
                                WHERE d.download_id = :id AND " . groupaccess( 'd.download_visibility' ) . ( multilang_table( 'DL' ) ? " AND " . in_group( 'dc.download_cat_language', LANGUAGE ) : '' ) . "
                                ORDER BY d.download_datestamp DESC", [':id' => $data['comment_item_id']]
                            );
                            if ( dbrows( $result_d ) ) {
                                $download_data = dbarray( $result_d );
                                $comment_start = dbcount( '(comment_id)', DB_COMMENTS, "comment_item_id = '" . $data['comment_item_id'] . "' AND comment_type = 'D' AND comment_id <= " . $data['comment_id'] );
                                $comment_start = $comment_start > $comments_per_page ? '&c_start_news_comments=' . ( ( floor( $comment_start / $comments_per_page ) * $comments_per_page ) - $comments_per_page ) : '';
                                $c_data += [
                                    'type'  => self::$locale['TMLN_092'],
                                    'link'  => INFUSIONS . 'downloads/downloads.php?download_id=' . $data['comment_item_id'] . $comment_start . '#c' . $data['comment_id'],
                                    'title' => $download_data['download_title']
                                ];
                            }
                        }
                        break;
                    case 'N':
                        if ( defined( 'NEWS_EXISTS' ) ) {
                            $result_n = dbquery( "SELECT n.news_subject
                                FROM " . DB_NEWS . " AS n
                                LEFT JOIN " . DB_NEWS_CATS . " AS nc ON nc.news_cat_id = n.news_cat
                                WHERE n.news_id = :id AND (n.news_start = 0 OR n.news_start <= '" . time() . "') AND (n.news_end = 0 OR n.news_end >= '" . time() . "') AND n.news_draft = 0 AND " . groupaccess( 'n.news_visibility' ) . ( multilang_table( 'NS' ) ? " AND " . in_group( 'n.news_language', LANGUAGE ) : '' ) . "
                                ORDER BY n.news_datestamp DESC", [':id' => $data['comment_item_id']]
                            );
                            if ( dbrows( $result_n ) ) {
                                $news_data = dbarray( $result_n );
                                $comment_start = dbcount( '(comment_id)', DB_COMMENTS, "comment_item_id = '" . $data['comment_item_id'] . "' AND comment_type = 'N' AND comment_id <= " . $data['comment_id'] );
                                $comment_start = $comment_start > $comments_per_page ? '&c_start_news_comments=' . ( ( floor( $comment_start / $comments_per_page ) * $comments_per_page ) - $comments_per_page ) : '';
                                $c_data += [
                                    'type'  => self::$locale['TMLN_093'],
                                    'link'  => INFUSIONS . 'news/news.php?readmore=' . $data['comment_item_id'] . $comment_start . '#c' . $data['comment_id'],
                                    'title' => $news_data['news_subject']
                                ];
                            }
                        }
                        break;
                    case 'P':
                        $result_p = dbquery( "SELECT p.photo_title
                            FROM " . DB_PHOTOS . " AS p
                            LEFT JOIN " . DB_PHOTO_ALBUMS . " AS pa USING (album_id)
                            WHERE p.photo_id = :id AND " . groupaccess( 'pa.album_access' ) . ( multilang_table( 'PG' ) ? " AND " . in_group( 'pa.album_language', LANGUAGE ) : '' ) . "
                            ORDER BY p.photo_datestamp DESC", [':id' => $data['comment_item_id']]
                        );
                        if ( dbrows( $result_p ) ) {
                            $photo_data = dbarray( $result_p );
                            $comment_start = dbcount( '(comment_id)', DB_COMMENTS, "comment_item_id = '" . $data['comment_item_id'] . "' AND comment_type = 'P' AND comment_id <= " . $data['comment_id'] );
                            $comment_start = $comment_start > $comments_per_page ? '&c_start_news_comments=' . ( ( floor( $comment_start / $comments_per_page ) * $comments_per_page ) - $comments_per_page ) : '';
                            $c_data += [
                                'type'  => self::$locale['TMLN_094'],
                                'link'  => INFUSIONS . 'gallery/gallery.php?photo_id=' . $data['comment_item_id'] . $comment_start . '#c' . $data['comment_id'],
                                'title' => $photo_data['photo_title']
                            ];
                        }
                        break;
                }
                $timeline[$data['comment_datestamp']] = [
                    'type'  => !empty($c_data['type']) ? $c_data['type'] : '',
                    'link'  => !empty($c_data['link']) ? $c_data['link'] : '',
                    'title' => !empty($c_data['title']) ? $c_data['title'] : '',
                    'time'  => $data['comment_datestamp']
                ];
            }

            if ( defined( 'FORUM_EXISTS' ) ) {
                $forum_activities = dbquery( "SELECT p.*, t.thread_subject, t.thread_author
                    FROM " . DB_FORUM_POSTS . " AS p
                    LEFT JOIN " . DB_FORUM_THREADS . " AS t ON t.thread_id = p.thread_id
                    LEFT JOIN " . DB_FORUMS . " AS f ON f.forum_id = p.forum_id
                    WHERE p.post_author = '" . $userid . "' AND " . groupaccess( 'forum_access' ) . "
                    ORDER BY p.post_id DESC, p.post_datestamp DESC
                ");
                while ( $data = dbarray( $forum_activities ) ) {
                    $first_post = dbarraynum( dbquery( "SELECT MIN(post_id) FROM " . DB_FORUM_POSTS . " WHERE thread_id = '" . intval( $data['thread_id'] ) . "' AND post_hidden = '0' GROUP BY thread_id" ) );
                    $first_post = $data['thread_author'] == $userid && $first_post[0] == $data['post_id'];
                    $timeline[$data['post_datestamp']] = [
                        'type'  => ($first_post ? self::$locale['TMLN_052'] : self::$locale['TMLN_053']),
                        'link'  => FORUM . 'viewthread.php?thread_id=' . $data['thread_id'] . '&pid=' . $data['post_id'] . '#post_' . $data['post_id'],
                        'title' => $data['thread_subject'] . ' #' . $data['post_id'],
                        'time'  => $data['post_datestamp']
                    ];
                }
            }
            if ( defined( 'TIMELINE_EXISTS' ) ) {
                $tline_p = dbquery( "SELECT t.*, tu.*
                    FROM " . DB_TIMELINE . " AS t
                    LEFT JOIN " . DB_USERS . " AS tu ON t.timeline_user = tu.user_id
                    WHERE t.timeline_user != '0' AND t.timeline_user = '" . $userid . "'" . ( multilang_table( 'TMLN' ) ? " AND " . in_group( 't.timeline_language', LANGUAGE ) : '' ) . "
                    ORDER BY t.timeline_datestamp DESC
                ");
                while ( $data = dbarray( $tline_p ) ) {
                    $timeline[$data['timeline_datestamp']] = [
                        'type'  => $data['user_name'] . self::$locale['TMLN_095'],
                        'link'  => '',
                        'title' => $data['timeline_description'],
                        'time'  => $data['timeline_datestamp']
                    ];
                }
            }

            if ( !empty( $this->timelinest['tline_autouser'] ) ) {
                $result_u = dbquery( "SELECT user_id, user_name, user_status, user_avatar, user_joined
                    FROM " . DB_USERS . "
                    WHERE user_id = '". $userid ."'
                " );
                if ( dbrows( $result_u ) ) {
                    $data = dbarray( $result_u );
                    $timeline[$data['user_joined']] = [
                        'type'  => sprintf( self::$locale['TMLN_096'], $data['user_name'] ),
                        'link'  => '',
                        'title' => sprintf( self::$locale['TMLN_097'], $data['user_name'] ),
                        'time'  => $data['user_joined']
                    ];
                }
            }
        }

        if ( !empty( $timeline ) ) {
            krsort( $timeline );
            foreach ( $timeline as $timestamp => $item ) {
                if ( !empty( $item['title'] ) ) {
                    $format = self::timelineFormat( $timestamp );
                    $_timeline[$format][] = $item;
                }
            }
        } else {
            $_timeline['nodata'] = self::$locale['TMLN_017'];
        }

        return $_timeline;
    }

    private static function timelineFormat( $time ) {
        if ( $time >= strtotime( 'today 00:00' ) ) {
            return showdate( 'G:i', $time );
        } else if ($time >= strtotime( 'yesterday 00:00' ) ) {
            return self::$locale['TMLN_018'];
        } else if ( $time >= strtotime( '-1 week 00:00' ) ) {
            return self::$locale['TMLN_019'];
        } else if ( $time >= strtotime( '-2 week 00:00' ) ) {
            return self::$locale['TMLN_020'];
        } else if ( $time >= strtotime( '-3 week 00:00' ) ) {
            return self::$locale['TMLN_021'];
        } else if ( $time >= strtotime('-1 Monday 00:00' ) ) {
            return self::$locale['TMLN_022'];
        } else {
            return showdate( 'Y M j', $time );
        }
    }
}